.if decpass

!======================================================
!
! File Enumerator Functions
!
! This file contains both the declarative and
! executable code required to enumerate the
! specified range of filenames from a specified
! Comet directory (QDIR)
!
! These functions use the MTB FSTAT functon to
! traverse the directory.  Therefore the information
! returned will coorespond to the information returned
! by the FSTAT function.  The resulting data is parsed
! and placed into variables associated with the
! respective data type.
!
! FS.FindFirst
! FS.FindNext
!
!======================================================
!Set FS.DEBUG = TRUE

! Parameters passed to the File Enumerator
Length 3 & Local FS.Dir$								! Directory to enumerate
Length 12 & Local FS.Start$, FS.End$				! Start and End filenames

! Resulting file information
Length 12 & Local FS.File$										! File ("" if End of directory)
Length 3 & Local FS.Ext$
Length 1 & Local FS.Type$
Length 1 & Local FS.Locked$
Length 2.0 & Local FS.KeySize
Length 3.0 & Local FS.OpenCount, FS.ExtractCount
Length 4.0 & Local FS.RecordSize
Length 10.0 & Local FS.FileSize, FS.KeyFileSize
Length 1.0 & Local FS.Err
Length 8 & Local FS.CreateDate$, FS.UpdateDate$
Length 6 & Local FS.UpdateTime$

! Internal constants and variables
Set FS.EndOfFile$ = "@FFFFFFFFFFFFFFFF@.@FFFFFF@"
Length 254 & Local FS.Stat$
Length 3 & Local FS.WorkDir$							! Directory to enumerate
Length 12 & Local FS.Next$, FS.Prev$

.else	! exepass

FS.FindFirst:
.if FS.DEBUG
	EXCP = 0
.endif
	FS.WorkDir$ = UCase(FS.Dir$)
	FS.Start$ = UCase(FS.Start$)
	FS.End$ = UCase(FS.End$)
	FS.File$ = ""

	! Makes for efficient checks
	If (FS.End$ EQ "") Then _
		FS.End$ = FS.EndOfFile$

	! Don't bother if out of range
	If (FS.End$ LT FS.Start$) Then _
		Goto FS.FirstExcp

	FS.Next$ = ""
	FS.Prev$ = ""
	FS.Stat$ = FStat(FS.Start$, Dir=FS.WorkDir$, Excp=FS.FirstExcp)
	Gosub FS.GetResult		! Get other info for this file

	! Did we find the start file?
	If (Sub(FS.Stat$, 3, 3) NE FS.WorkDir$)
.if FS.DEBUG
		print "FS.Start$ NOT found"
.endif
		Gosub FS.FindNext			! No - get next
	Else
.if FS.DEBUG
		print "FS.Start$ found"
.endif
	EndIf

	Goto FS.FirstDone

FS.FirstExcp:
.if FS.DEBUG
	If (EXCP NE 0) Then _
		Print "Excp:";EXCP
.endif
	FS.Next$ = ""
	FS.File$ = ""

FS.FirstDone:
Return	! FS.FindFirst

FS.FindNext:
.if FS.DEBUG
	EXCP = 0
.endif
	FS.File$ = ""

	! Can't come here if FindFirst not called
	If (FS.Next$ EQ "")
		print "(FS.Next$ EQ '')"
		Goto FS.NextExcp
	EndIf

	! Didn't advance for some reason - Abort
	If (FS.Prev$ EQ FS.Next$)
.if FS.DEBUG
		print "(FS.Prev$ EQ FS.Next$)"
.endif
		Goto FS.NextExcp
	EndIf

	! End of file - Abort
	If (FS.Next$ EQ FS.EndOfFile$)
.if FS.DEBUG
		print "(FS.Next$ EQ FS.EndOfFile$)"
.endif
		Goto FS.NextExcp
	EndIf

	! End of range - Abort
	If (FS.Next$ GT FS.End$)
.if FS.DEBUG
		print "(FS.End$ NE '' And FS.Next$ GT FS.End$)"
.endif
		Goto FS.NextExcp
	EndIf

	FS.Prev$ = FS.Next$	! For the record
	FS.File$ = FS.Next$	! For returning
	FS.Stat$ = FStat(FS.Next$, Dir=FS.WorkDir$, Excp=FS.NextExcp)
	Gosub FS.GetResult		! Get other info for this file
	Goto FS.NextDone

FS.NextExcp:
	FS.Next$ = ""
	FS.File$ = ""
.if FS.DEBUG
	If (EXCP NE 0) Then _
		print "EXCP:";EXCP
.endif

FS.NextDone:
Return	! FS.FindNext

FS.GetResult:
	FS.Next$ = StripR(Sub(FS.Stat$, 35, 8))				! Next filename
	FS.Ext$ = StripR(Sub(FS.Stat$, 43, 3))					! Extension
	If (FS.Ext$ NE "") Then _									! Extension supplied
		FS.Next$ = FS.Next$ + "." + FS.Ext$

	FS.File$ = StripR(Sub(FS.Stat$, 50, 8))				! Translated filename
	FS.Ext$ = StripR(Sub(FS.Stat$, 59, 3))					! Extension
	If (FS.Ext$ NE "") Then _									! Extension supplied
		FS.File$ = FS.File$ + "." + FS.Ext$

	FS.Dir$ = Sub(FS.Stat$, 3, 3)
	FS.Type$ = Sub(FS.Stat$, 1, 1)							! "K","D", "T", "O", "Q"
	FS.Locked$ = Sub(FS.Stat$, 2, 1)							! "P", " "
	FS.OpenCount = Num(Sub(FS.Stat$, 6, 3), FS.Err)		! nnn.0
	FS.ExtractCount = Num(Sub(FS.Stat$, 9, 3), FS.Err)	! nnn.0
	FS.RecordSize = Num(Sub(FS.Stat$, 46, 4), FS.Err)	! nnnn.0
	FS.FileSize = HexDec(Sub(FS.Stat$, 15, 4))			! nnnnnnnnnn.0
	FS.KeySize = Num(Sub(FS.Stat$, 31, 2), FS.Err)		! nn.0
	FS.KeyFileSize = HexDec(Sub(FS.Stat$, 135, 4))		! nnnnnnnnnn.0
	FS.CreateDate$ = Sub(FS.Stat$, 62, 8)					! Create date
	FS.UpdateDate$ = Sub(FS.Stat$, 70, 8)					! Update date
	FS.UpdateTime$ = Sub(FS.Stat$, 131, 4) +_				! Update time HHMM
						  Sub(FS.Stat$, 129, 2)					! Update time SS
Return	! FS.GetResult
.endif	! exepass

